// Widget to connect to broadway's avc

var wasmTest = function() {
    try {
        if (typeof WebAssembly === "object"
            && typeof WebAssembly.instantiate === "function") {
            const module = new WebAssembly.Module(Uint8Array.of(0x0, 0x61, 0x73, 0x6d, 0x01, 0x00, 0x00, 0x00));
            if (module instanceof WebAssembly.Module)
                return new WebAssembly.Instance(module) instanceof WebAssembly.Instance;
        }
    } catch (e) {
    }
    return false;
};
var wasmSupported = wasmTest();

// help function to turn array into string for printing
function Ui8ArraytoStr(array, length){
    var out = "";    //output as string
    var c;

    for (var i = 0; i < length; i++) {
        c = array[i];
        if(c < 16)
            out += '0';
        out += c.toString(16);
        out += ' ';
    };

    return out;
}


// returns postion of multi-byte pattern in array
// returns 0 if not found, otherwise position of pattern in array
// 1 (not 0) = first position in array
function Ui8ArrayFind(array, pattern)
{

    var offset = 0;
    var patternNotFound = 1;

    var length = array.length

    // outer loop, go through all bytes in array
    while (offset < length && patternNotFound) {
        var j=0;
        // inner loop, test whether position in array holds requested pattern
        while(offset + j < length && array[offset + j] == pattern[j]) {
            j++;
        }

        // pattern not found yet ?
        patternNotFound = (j != pattern.length);
        offset++;
    };

    if (patternNotFound)
        return 0;
    else
        return offset;

}

// Widget to connect to broadway's avc
function AvcPlayer(canvas, rendermode){

    var self = this;
    var defaultConfig = {
        filter: "original",
        filterHorLuma: "optimized",
        filterVerLumaEdge: "optimized",
        getBoundaryStrengthsA: "optimized"
    }

    //allocate static memory for pixel data (either webgl or 2D)
    this.setup = function(canvas, rendermode) {
        if (rendermode == 'webgl') {
            this.webGLCanvas = new YUVWebGLCanvas(canvas, new Size(canvas.width, canvas.height));
            // Set the viewport to match new size
            try {
                gl = canvas.getContext("experimental-webgl") || canvas.getContext("webgl");
            } catch(err) {
                logger.error("Error: canvas does not have a webgl context");
            }
            if (gl) {
                gl.viewport(0, 0, canvas.width, canvas.height);
            }
        } else {
            this.ctx = canvas.getContext("2d");
            this.RGBAdata = self.ctx.createImageData(canvas.width, canvas.height);
            for(var i = 0; i < this.RGBAdata.data.length; i++) {
                this.RGBAdata.data[i] = 255; //set all channels to opaque
            }
        }
    }
    this.setup(canvas, rendermode);

    logger.info("wa: " + wasmSupported + ", rendermode: " + rendermode);
    //set up avc
    if (wasmSupported) {
        this.avc = new Decoder();
    } else {
        this.avc = new Avc();
        this.avc.configure(defaultConfig);
    }
    this.avc.onPictureDecoded = function(buffer, width, height) {

        if (!buffer) {
            logger.debug("AvcPlayer:onPictureDecoded buffer empty");
            return;
        };

        var lumaSize = width * height;
        var chromaSize = lumaSize >> 2;

        // webgl ok ? -> use webgl renderer
        var webGLCanvas = self.webGLCanvas;
        if (webGLCanvas) {
            webGLCanvas.YTexture.fill(buffer.subarray(0, lumaSize));
            webGLCanvas.UTexture.fill(buffer.subarray(lumaSize, lumaSize + chromaSize));
            webGLCanvas.VTexture.fill(buffer.subarray(lumaSize + chromaSize, lumaSize + 2 * chromaSize));
            webGLCanvas.drawScene();
        } else {
            //webgl not ok -> use 2D renderer
            var ytexture = buffer.subarray(0, lumaSize);
            var utexture = buffer.subarray(lumaSize, lumaSize + chromaSize);
            var vtexture = buffer.subarray(lumaSize + chromaSize, lumaSize + 2 * chromaSize);

            var posn = 0;
            var ypos = 0;

            var data = self.RGBAdata.data;

            var arraypos = 0;
            for (var y = 0; y < height; y++) {
                var ypos = (y >> 1) * (width >> 1);
                for (var x = 0; x < width; x += 4) {
                    var t0 = ytexture[arraypos >> 2];
                    var t1 = ytexture[(arraypos >> 2) + 1];
                    var t2 = ytexture[(arraypos >> 2) + 2];
                    var t3 = ytexture[(arraypos >> 2) + 3];

                    var posn = ypos + (x >> 1);
                    var YUV444_Y = (t0 - 16) * 298;
                    var YUV444_U = utexture[posn] - 128;
                    var YUV444_V = vtexture[posn] - 128;
                    var A = (409 * YUV444_V);
                    var B = (100 * YUV444_U) + (208 * YUV444_V);
                    var C = (516 * YUV444_U);
                    data[arraypos] = (YUV444_Y + A + 128) >> 8; // Red
                    data[arraypos + 1] = (YUV444_Y - B + 128) >> 8; // Green
                    data[arraypos + 2] = (YUV444_Y + C + 128) >> 8; // Blue

                    YUV444_Y = (t1 - 16) * 298;
                    data[arraypos + 4] = (YUV444_Y + A + 128) >> 8; // Red
                    data[arraypos + 5] = (YUV444_Y - B + 128) >> 8; // Green
                    data[arraypos + 6] = (YUV444_Y + C + 128) >> 8; // Blue

                    posn = ypos + ((x + 2) >> 1);
                    YUV444_Y = (t2 - 16) * 298;
                    YUV444_U = utexture[posn] - 128;
                    YUV444_V = vtexture[posn] - 128;
                    A = (409 * YUV444_V);
                    B = (100 * YUV444_U) + (208 * YUV444_V);
                    C = (516 * YUV444_U);
                    data[arraypos + 8] = (YUV444_Y + A + 128) >> 8; // Red
                    data[arraypos + 9] = (YUV444_Y - B + 128) >> 8; // Green
                    data[arraypos + 10] = (YUV444_Y + C + 128) >> 8; // Blue

                    YUV444_Y = (t3 - 16) * 298;
                    data[arraypos + 12] = (YUV444_Y + A + 128) >> 8; // Red
                    data[arraypos + 13] = (YUV444_Y - B + 128) >> 8; // Green
                    data[arraypos + 14] = (YUV444_Y + C + 128) >> 8; // Blue
                    arraypos += 16;
                }
            }
            self.ctx.putImageData(self.RGBAdata, 0, 0);
        }
    };

    this.decode = function(data) {
        this.avc.decode(data);
    };
};

function setupCanvas(div) {
    var canvas = document.createElement('canvas');

    width = div.attributes.width ? div.attributes.width.value : 640;
    height = div.attributes.height ? div.attributes.height.value : 480;

    canvas.width = width;
    canvas.height = height;
    canvas.style.backgroundColor = "#996633";
    div.appendChild(canvas);

    webGLCanvas = new YUVWebGLCanvas(canvas, new Size(width, height));
}
