var guestAlertCount = 0;
var guestAlertIDs = {};
var registerGuestAlert = function(name) {
    guestAlertCount += 1;
    var id = "guest-alert-" + guestAlertCount;
    guestAlertIDs[name] = id;
    return id;
}

function GuestAlertMenu(client, id) {
    this.client = client;
    this.id = id;
    this.element = document.getElementById(this.id);
    this.guestAlerts = {};  // key: Usernames, value: GuestAlert objects

    this.handleGuestSignInReply = function(jsonMessage) {
        var guestName = jsonMessage[XRM_MU_SVR_GUEST_SIGNIN_CLIENT_NAME];
        var isAccepted = false;

        isAccepted = !!(jsonMessage[XRM_MU_SVR_GUEST_SIGNIN_ACCEPTED]);

        if (this.guestAlerts[guestName]) {
            this.guestAlerts[guestName].handleResponse(isAccepted);

            if ($(this.element).find("div.guest-pending").length === 0) {
                $(this.element).fadeOut();
                for (var g in this.guestAlerts) {
                    $(this.element).find("div.guest-alert").remove();
                    delete this.guestAlerts[g.__id];
                    delete g;
                }
            }
        }
    }.bind(this);

    this.handleConfirmGuestRequest = function(jsonMessage) {
        logger.debug("Received confirm guest message");

        var inviteKey = jsonMessage[XRM_MU_SVR_CONFIRM_INVITE_UUID];
        var guestName = jsonMessage[XRM_MU_SVR_CONFIRM_INVITE_NAME];

        if ($("#" + guestAlertIDs[guestName]).length == 0) {
            var ga = new GuestAlert(this.element, guestName, inviteKey, client.dataWS)
            this.guestAlerts[ga.__name] = ga;
            $("#" + this.id).slideDown();
        }
    }.bind(this);
}

//
// Adds a GuestAlert row to parent.
//
// parent = the guest alert menu DOM element
// name = the name of the guest
// key = the invite key
// websocket = the data websocket
//
function GuestAlert(parent, name, key, websocket) {
    this.__parent = parent;
    this.__id = registerGuestAlert(name);
    this.__name = name;
    this.__inviteKey = key;
    this.__websocket = websocket;
    this.__isAccepted = null;

    this.__template =
        '<div id="' + this.__id + '" class="message guest-alert guest-pending">' +
        '  <div class="message-text"><span class="glyphicon glyphicon-user"></span> "' + htmlEncode(this.__name) + '" wants to join.</div>  <div class="guest-join-status hidden"><span class="guest-join-status-icon"></span></div><button class="btn btn-med btn-default guest-decline" disabled="true">Decline</button><button class="btn btn-med btn-default guest-accept" disabled="true">Accept</button>' +
        '</div>';

    $(this.__parent).append(this.__template);
    setTimeout(function() {
        $("#" + this.__id).find(".btn").prop("disabled", false);
    }.bind(this), 1000);

    this.handleResponse = function(state) {
        this.__isAccepted = state;
        var $div = $("#" + this.__id);
        $div.find(".btn").remove();
        $div.find(".guest-join-status:first").removeClass("hidden");
        if (this.__isAccepted) {
            $div.find(".guest-join-status-icon:first").addClass("fas fa-thumbs-up fa-lg");
        } else {
            $div.find(".guest-join-status-icon:first").addClass("fas fa-ban fa-lg");
        }

        $div.removeClass("guest-pending");
    }.bind(this);

    this.sendResponse = function(accepted) {
        var val = 0;

        if (accepted === true)
            val = 1;

        var obj = {};
        obj[XRM_MSG_TYPE] = XRM_MSGTYPE_MU_CLNT_CONFIRM_INVITE;
        obj[XRM_MU_CLNT_CONFIRM_INVITE_NAME] = this.__name;
        obj[XRM_MU_CLNT_CONFIRM_INVITE_UUID] = this.__inviteKey;
        obj[XRM_MU_CLNT_CONFIRM_INVITE_VAL] = val;
        this.__websocket.send(JSON.stringify(obj));
    }.bind(this);

    $("div.guest-alert:last .guest-decline").on("click", function(e) {
        this.handleResponse(false);
        this.sendResponse(false);
    }.bind(this));

    $("div.guest-alert:last .guest-accept").on("click", function(e) {
        this.handleResponse(true);
        this.sendResponse(true);
    }.bind(this));

    return this;
};
