// kbm-events.js
//
// Serialize keyboard and mouse events on an html page into JSON
// Access functions with singleton object 'KBMEvents'
//
function XrmKBMEvents()
{
    this.serialMouseEvents = function(event, canvas, screenID) {
        // return null if canvas isn't ready
        if (canvas.offsetWidth === 0) {
            return null;
        }

        var normX = (event.pageX - canvas.offsetLeft);
        var normY = (event.pageY - canvas.offsetTop);
        if (typeof myScrollbars !== "undefined") {
            var scrollInfo = myScrollbars.scroll();
            normX = normX + scrollInfo.x.position;
            normY = normY + scrollInfo.y.position;
        }

        /**
         * Fixes old-Firefox fullscreen coordinates mapping
         */
        if (canvas.width !== canvas.offsetWidth || canvas.height !== canvas.offsetHeight) {
            normX = normX * canvas.width / canvas.offsetWidth;
            normY = normY * canvas.height / canvas.offsetHeight;
        }

        /**
         * Standardizing mouse wheel values is tricky.
         *
         * There are three dimensions to this:
         * (1) Server operating system.  The server tells this to us.
         * (2) Client operating system + Client browser.  We detect this by looking at
         *       the attributes available in the event.
         * (3) Device.
         *
         * Notes:
         * - Linux / GtK has 53 as its default scroll tick.  Windows has a scroll tick of 120.
         * - Chrome 49 sets wheelDelta to 120 for mice in Win/Linux and 12 in Safari 9.
         * - Firefox 45 uses a deltaMode of 1, EXCEPT when using a trackpad in OS X.
         * - IE 11 uses a deltaMode of 1.
         */
        var normDeltaY = 0;

        if (event.deltaY != null) {
            normDeltaY = event.deltaY;
            if (!this.LINE_PIXELS) {
                if (serverOS == "Linux") {
                    this.LINE_PIXELS = 53;
                } else if (serverOS == "Windows") {
                    this.LINE_PIXELS = 120;
                } else {
                    this.LINE_PIXELS = 120;
                }
                this.PAGE_PIXELS = 800;
            }

            if (event.wheelDelta) { // Chrome, Safari
                if (Math.abs(event.wheelDelta) === 12 || Math.abs(event.wheelDelta) === 120) {
                    normDeltaY = event.wheelDelta > 0 ? -this.LINE_PIXELS : this.LINE_PIXELS;
                }
            } else if (event.deltaMode === 1) { // Firefox, IE
                normDeltaY = event.deltaY > 0 ? this.LINE_PIXELS : -this.LINE_PIXELS;
            } else if (event.deltaMode === 2) { // Not sure if anything uses this
                normDeltaY = event.deltaY > 0 ? this.PAGE_PIXELS : -this.PAGE_PIXELS;
            }
        }

        var mods = 0;

        if (event.altKey)
            mods = mods | XRM_CLNT_KBM_ALT;
        if (event.ctrlKey)
            mods = mods | XRM_CLNT_KBM_CTRL;
        if (event.metaKey)
            mods = mods | XRM_CLNT_KBM_META;
        if (event.shiftKey)
            mods = mods | XRM_CLNT_KBM_SHIFT;

        var obj = {};
        obj[XRM_MSG_TYPE] = XRM_MSGTYPE_CLNT_MOUSE;
        obj[XRM_MSG_SCREEN_ID] = screenID;
        obj[XRM_CLNT_MOUSE_EVT] = event.type;
        obj[XRM_CLNT_MOUSE_MOD] = mods;
        obj[XRM_CLNT_MOUSE_BUTTON] = event.button;
        obj[XRM_CLNT_MOUSE_X] = normX;
        obj[XRM_CLNT_MOUSE_Y] = normY;
        obj[XRM_CLNT_MOUSE_DY] = normDeltaY;

        return (JSON.stringify(obj));
    }.bind(this);

    this.serialKeyEvents = function(event) {
        var mods = 0;

        if (event.altKey)
            mods = mods | XRM_CLNT_KBM_ALT;
        if (event.ctrlKey)
            mods = mods | XRM_CLNT_KBM_CTRL;
        if (event.metaKey)
            mods = mods | XRM_CLNT_KBM_META;
        if (event.shiftKey)
            mods = mods | XRM_CLNT_KBM_SHIFT;

        var obj = {};
        obj[XRM_MSG_TYPE] = XRM_MSGTYPE_CLNT_KEY;
        obj[XRM_CLNT_KEY_EVT] = event.type;
        obj[XRM_CLNT_KEY_CHARCODE] = event.charCode;
        obj[XRM_CLNT_KEY_KEYCODE] = event.keyCode;
        obj[XRM_CLNT_KEY_MOD] = mods;

        return (JSON.stringify(obj));
    }.bind(this);
}

var KBMEvents = new XrmKBMEvents();
