/**
 *
 * NOTE: Depends on global variable
 *
 *   myScreenID
 *
 */

function XrmKeyboardMouse(canvas, dataWS, enabled)
{
    $(canvas).after("<textarea class='xrm-input' autofocus></textarea>");
    this.input = $(canvas).next(".xrm-input")[0];
    this.canvas = canvas;
    this.dataWS = dataWS;

    var clipboardBuffer = null;

    /**
     * KBM handling
     **/
    if (enabled) {
        var lastKeydown = null;
        var keydownDuplicateCount = 0;
        var lastMousedownTarget = null;

        var clearKeyEvent = function(event) {
            if (myScreenID === null)
                return false;

            if (!KeyboardMouseActive) {
                return true;
            }
            event.returnValue = false;
            var obj = {};
            obj[XRM_MSG_TYPE] = XRM_MSGTYPE_CLNT_BLUR;
            this.dataWS.send(JSON.stringify(obj));
            return false;
        }.bind(this);

        var handleOSXCapsLockEvent = function(event) {
            // Issue #135: OS X capslock key presses now always produce a keydown
            //             event and a keyup event
            if (event.keyCode === 20 && clientOS === "MacOS") {
                var forcedKeydown = jQuery.extend({}, event);
                forcedKeydown.type = "keydown";
                forcedKeydown.keyCode = event.keyCode;
                if (DEBUG_KBM_EVENTS) {
                    logger.debug("Forced keyevnts =", KBMEvents.serialKeyEvents(forcedKeydown));
                }
                this.dataWS.send(KBMEvents.serialKeyEvents(forcedKeydown));
                var forcedKeyup = jQuery.extend({}, event);
                forcedKeyup.type = "keyup";
                forcedKeyup.keyCode = event.keyCode;
                if (DEBUG_KBM_EVENTS) {
                    logger.debug("Forced keyevnts =", KBMEvents.serialKeyEvents(forcedKeyup));
                }
                this.dataWS.send(KBMEvents.serialKeyEvents(forcedKeyup));
                return true;
            }
            return false;
        }.bind(this);

        var handleOSXCommandEvent = function(event, tmpKeydown) {
            // Issue #72: In OS X clients, mixing command key with other keydown events
            //            sometimes fails to produce keyup
            if ((event.keyCode === 91 || event.keyCode === 93 || event.keyCode === 224) &&
                event.type === "keyup" && clientOS === "MacOS" && serverOS === "Linux") {

                if (tmpKeydown != null && tmpKeydown != 20 && tmpKeydown != 144 && tmpKeydown != 145) { // exception for capslock
                    // Force a keyup event to match the last keydown
                    var forcedKeyup = jQuery.extend({}, event);
                    forcedKeyup.type = "keyup";
                    forcedKeyup.keyCode = tmpKeydown;

                    if (DEBUG_KBM_EVENTS)
                        logger.debug("Forced keyevnts =", KBMEvents.serialKeyEvents(forcedKeyup));

                    this.dataWS.send(KBMEvents.serialKeyEvents(forcedKeyup));
                    return true;
                }
            }
            return false;
        }.bind(this);

        var sendFocusEvent = function(event) {
            if (myScreenID === null)
                return false;

            if (this.dataWS === undefined) {
                window.setTimeout(function() { sendFocusEvent(event); }, 1000);
            } else {
                var obj = {};
                obj[XRM_MSG_TYPE] = XRM_MSGTYPE_CLNT_FOCUS;
                this.dataWS.send(JSON.stringify(obj));
            }
            return true;
        }.bind(this);

        var delayedKeyEvents = [];
        var delaySendKeyEvent = function(event) {
            delayedKeyEvents.push(event);
            setTimeout(function() {
                var e = delayedKeyEvents.shift();
                sendKeyEvent(e, true);
            }.bind(this), 0);
        }.bind(this);

        var isAltDown = false;
        var isCtrlDown = false;
        var isMetaDown = false;
        var isShiftDown = false;

        var sendKeyEvent = function(event, delayed) {
            isAltDown = event.altKey;
            if (event.type == "keyup" && event.keyCode == 18)
                isAltDown = false;

            isCtrlDown = event.ctrlKey;
            if (event.type == "keyup" && event.keyCode == 17)
                isCtrlDown = false;

            isShiftDown = event.shiftKey;
            if (event.type == "keyup" && event.keyCode == 16)
                isShiftDown = false;

            isMetatDown = event.metaKey;
            if (event.type == "keyup" && ((event.keyCode == 91 || event.keyCode == 92) || (event.keyCode == 93 && clientOS == "MacOS")))
                isMetaDown = false;

            if (myScreenID === null)
                return false;

            this.input.value = "";

            // HACK: ctrl-v events set "delayed = true" because we need them to happen AFTER
            //       the paste event.  All key events must wait until the ctrl-v event is
            //       processed.  pasteEnabled comes from server-config.js
            if (pasteEnabled === true && delayed !== true) {
                if (event.ctrlKey === true && event.keyCode === 86) {
                    delaySendKeyEvent(event);
                    event.returnValue = true; // NOTE: this allows paste event to fire
                    return true;
                } else if (delayedKeyEvents.length > 0) {
                    delaySendKeyEvent(event);
                    event.returnValue = false;
                    event.preventDefault();
                    return false;
                }
            }

            event.returnValue = false; // This must come after Ctrl-V capture or paste event won't happen in Chrome

            if (!KeyboardMouseActive) {
                return true;
            }

            if (DEBUG_KBM_EVENTS) {
                logger.debug("Keyevnts = ", KBMEvents.serialKeyEvents(event), ", _delayed = ", event._delayed);
            }

            var sendEvent = true;
            var tmpKeydown = lastKeydown;
            if (serverOS === "Linux" && event.type === "keydown") {
                if (event.keyCode === lastKeydown && keydownDuplicateCount < 10) {
                    sendEvent = false;
                    keydownDuplicateCount += 1;
                } else {
                    keydownDuplicateCount = 0;
                }
                lastKeydown = event.keyCode;
            } else {
                lastKeydown = null;
            }

            if (sendEvent === true) {
                if (handleOSXCapsLockEvent(event) === false) {
                    this.dataWS.send(KBMEvents.serialKeyEvents(event));
                    handleOSXCommandEvent(event, tmpKeydown);
                }
            }

            /* Stop special browser keys from opening browser controls */
            event.preventDefault();

            /* Stop alt, ctrl, meta modified keys from opening controls */
            if (event.altKey === true || event.ctrlKey === true || event.metaKey === true) {
                return false;
            }
        }.bind(this);

        var sendMouseEvent = function(event) {
            // logger.debug("wheel deltaY = " + event.deltaY + ", detail = " + event.detail + ", wheelDelta = " + event.wheelDelta + ", deltaMode = " + event.deltaMode);
            if (myScreenID === null) {
                return false;
            }

            if (!KeyboardMouseActive) {
                return true;
            }

            /**
             * Don't interfere with mouse events on 'input' elements
             * (this breaks Chrome input of type 'number')
             */
            var eventTagName = event.target.tagName;
            if (eventTagName && eventTagName.toLowerCase() === "input") {
                return true;
            }

            if (event.type === "mousedown") {
                lastMousedownTarget = event.target.id;
            }

            /**
             * Only transmit mousedown, clicks, and double-click
             * events if they happened directly on the canvas.
             */
            if (event.target.id !== this.canvas.id && (event.type === "mousedown" || event.type === "click" || event.type === "dblclick")) {
                return true;
            }

            /**
             * mouseup events are sent if they started in the canvas
             * or from undefined target
             */
            if (event.type === "mouseup") {
                var allowMouseup = (lastMousedownTarget === this.canvas.id || lastMousedownTarget === null);
                lastMousedownTarget = null;
                if (!allowMouseup)  {
                    return true;
                }
            }

            var mouseMessage = KBMEvents.serialMouseEvents(event, this.canvas, myScreenID);
            if (DEBUG_KBM_EVENTS) {
                logger.debug("Mouseevnts = ", mouseMessage);
            }
            if (mouseMessage === null) {
                return;
            }

            this.dataWS.send(mouseMessage);

            event.preventDefault();
        }.bind(this);

        // add mouse listeners
        window.addEventListener("click", sendMouseEvent, true);
        window.addEventListener("dblclick", sendMouseEvent, true);
        window.addEventListener("mousedown", sendMouseEvent, true);
        window.addEventListener("mousemove", sendMouseEvent, true);
        window.addEventListener("mouseup", sendMouseEvent, true);
        addWheelListener(this.canvas, sendMouseEvent, true);

        // prevent right click to open context menu on browser
        this.canvas.addEventListener("contextmenu", function(event) {
            event.preventDefault();
            return false;
        }.bind(this), false);

        // clicking anywhere puts focus on canvas so key input is not interrupted
        $("html").on("click", function(event) {
            /* Don't interfere with 'input' elements */
            var eventTagName = event.target.tagName;
            if (eventTagName && eventTagName.toLowerCase() === "input") {
                return true;
            }
            $(this.input).focusNoScroll();
        }.bind(this));

        // KeyEvents aren't fired on canvases w/o tabindex,
        // see http://www.dbp-consulting.com/tutorials/canvas/CanvasKeyEvents.html
        this.input.setAttribute("tabindex", "0");
        $(this.input).focusNoScroll();

        // add key listeners
        this.input.addEventListener("focus", sendFocusEvent, false);
        this.input.addEventListener("blur", clearKeyEvent, false);
        this.input.addEventListener("keydown", sendKeyEvent, false);
        this.input.addEventListener("keyup", sendKeyEvent, false);
        document.addEventListener("keydown", function(event) {
            if (event.ctrlKey === true && event.keyCode === 123) {
                $(".menubar-toggleable").toggle();
            }
        }.bind(this), false);

        document.addEventListener("paste", function(event) {
            var obj = {};
            obj[XRM_MSG_TYPE] = XRM_MSGTYPE_CLNT_PASTE;

            var data;
            if (window.clipboardData) {
                data = window.clipboardData.getData("Text");
            } else {
                data = event.clipboardData.getData("text/plain");
            }

            obj[XRM_CLNT_PASTE_VAL] = data;

            var msg = JSON.stringify(obj);
            // Truncate paste data that's too big
            if (msg.length > MAX_X11REMOTE_FRAME_SIZE) {
                var jsonOverhead = msg.length - data.length;
                data = data.slice(0, MAX_X11REMOTE_FRAME_SIZE - jsonOverhead);
                obj[XRM_CLNT_PASTE_VAL] = data;
                msg = JSON.stringify(obj);
            }

            if (clipboardBuffer !== data) {
                clipboardBuffer = data;
                this.dataWS.send(msg);
            }
        }.bind(this), false);  // all browsers and IE9+

        this.__updateCursor = function(cursorName) {
            var cursorURL = location.origin + "/cursors/" + cursorName;
            if (cursorName === XRM_SVR_LOCALCURSOR_NULL) {
                this.canvas.style.cursor = "none";
            } else {
                this.canvas.style.cursor = "url('" + cursorURL + "'), auto";
            }
            logger.trace("Cursor URL: " + cursorURL);
        }.bind(this);

        this.handleCursorChange = function(cursorName) {
            logger.debug("Received Cursor Update");
            this.__updateCursor(cursorName);
        }.bind(this);

        this.__setAutoCursor = function() {
            this.canvas.style.cursor = "auto";
            logger.trace("Set auto-cursor");
        }.bind(this);

        this.setAutoCursor = function() {
            this.__setAutoCursor();
        }.bind(this);

        document.getElementById("ctrlaltdel").onclick = function(e) {
            if (myScreenID === null)
                return false;
            var obj = {};
            obj[XRM_MSG_TYPE] = XRM_MSGTYPE_CLNT_CTRLALTDEL;
            this.dataWS.send(JSON.stringify(obj));
            return false;
        }.bind(this);

        document.getElementById("printscreen").onclick = function(e) {
            if (myScreenID === null)
                return false;

            var event = {};
            event.altKey = isAltDown;
            event.ctrlKey = isCtrlDown;
            event.shiftKey = isShiftDown;
            event.metaKey = isMetaDown;

            // HACK: Linux needs an extra 'alt-keydown' event just before
            // printscreen or it doesn't register.
            if (isAltDown === true) {
                event.type = "keydown";
                event.charCode = 18;
                event.keyCode = 18;
                this.dataWS.send(KBMEvents.serialKeyEvents(event));
            }

            // The corresponding keyup event for printscreen is handled by the server
            event.type = "keydown";
            event.charCode = 44;
            event.keyCode = 44;
            this.dataWS.send(KBMEvents.serialKeyEvents(event));

            return false;
        }.bind(this);
    }
}
