/**
 * Normalize mouse-wheel handling.  'wheel' is the new standard event
 * and should be used whenever possible, but older browsers handled
 * this with 'onwheel', 'mousewheel', or 'DOMMouseScroll'.
 *
 * Public call:
 * addWheelListener(elem, callback, useCapture);
 *
 * This code was taken from:
 * https://developer.mozilla.org/en-US/docs/Web/Events/wheel
 *
 **/
var prefix = "", _addEventListener, onwheel, support;

// detect event model
if (window.addEventListener) {
    _addEventListener = "addEventListener";
} else {
    _addEventListener = "attachEvent";
    prefix = "on";
}

// detect available wheel event
support = "onwheel" in document.createElement("div") ? "wheel" : // Modern browsers support "wheel"
          document.onmousewheel !== undefined ? "mousewheel" : // Webkit and IE support at least "mousewheel"
          "DOMMouseScroll"; // let's assume that remaining browsers are older Firefox

window.addWheelListener = function(elem, callback, useCapture) {
    _addWheelListener(elem, support, callback, useCapture);

    // handle MozMousePixelScroll in older Firefox
    if (support == "DOMMouseScroll") {
        _addWheelListener(elem, "MozMousePixelScroll", callback, useCapture);
    }
};

function _addWheelListener(elem, eventName, callback, useCapture) {
    var options = useCapture;
    if (passiveSupported) {
        // As of Chrome 73, any targets of the wheel event default to
        // being 'passive'.  Passive targets can't call preventDefault(),
        // which can lead to undesired scrolling.  To fix this we set
        // passive back to false.
        options = {capture: useCapture, passive: false};
    }

    elem[ _addEventListener ](prefix + eventName, support == "wheel" ? callback : function(originalEvent) {
        !originalEvent && (originalEvent = window.event);

        // create a normalized event object
        var event = {
            // keep a ref to the original event object
            originalEvent: originalEvent,
            target: originalEvent.target || originalEvent.srcElement,
            type: "wheel",
            deltaMode: originalEvent.type == "MozMousePixelScroll" ? 0 : 1,
            deltaX: 0,
            deltaZ: 0,
            altKey: originalEvent.altKey,
            ctrlKey: originalEvent.ctrlKey,
            metaKey: originalEvent.metaKey,
            shiftKey: originalEvent.shiftKey,
            button: originalEvent.button,
            pageX: originalEvent.pageX,
            pageY: originalEvent.pageY,
            preventDefault: function() {
                originalEvent.preventDefault ?
                    originalEvent.preventDefault() :
                    originalEvent.returnValue = false;
            }
        };

        // calculate deltaY (and deltaX) according to the event
        if (support == "mousewheel") {
            event.deltaY = - 1/40 * originalEvent.wheelDelta;
            // Webkit also support wheelDeltaX
            originalEvent.wheelDeltaX && (event.deltaX = - 1/40 * originalEvent.wheelDeltaX);
        } else {
            event.deltaY = originalEvent.detail;
        }

        // it's time to fire the callback
        return callback(event);

    }, options || false);
}
