var gNativeClientEnabled = false;

var gJSBridgeNonce;

function NativeClient() {
    ///// Private
    var enabled = false;                  /* true only if the native decoder and browser is being used */
    var nativeClientServerPort = 54234;
    var nativeClientWS;

    var nativeClientVersion;

    var initialized = false;
    var connected = false;
    var synchronized = false;

    var ncLosslessEnabled = false;
    var ncVirtualHereSupport = false;

    var lastVHDeviceListMsg = "";

    ///// Public
    this.constructor = function() {
        nativeClientVersion = "-2,-2";
        if (NativeClient.isNativeDecoderEnabled()) {
            if (navigator.userAgent.indexOf("decoder:Native;") > 0) {
                var clVer = navigator.userAgent.substring(0, navigator.userAgent.indexOf(" "));
                nativeClientVersion = clVer.substring(clVer.indexOf("/") + 1);
            }
            logger.info("Using Native decoder, version " + nativeClientVersion);
            enabled = true;
            var regex = /bridgePort:([0-9]*)/;
            nativeClientServerPort = navigator.userAgent.match(regex)[1]; // port-string
        } else {
            logger.info("Using Web decoder");
        }
        initialized = true;
    }

    this.openNativeClientConnection = function(clientID) {
        if (enabled === true) {
            logger.log("NativeClient: Opening js-bridge on port " + nativeClientServerPort);
            this.nativeClientWS = new WebSocket("ws://127.0.0.1:" + nativeClientServerPort + "/ws/js-bridge/" + clientID + "/" + gJSBridgeNonce);
            logger.log("NativeClient: js-bridge opened");
            var ncWSConnectedWait = setInterval(function() {
                var readyState = this.nativeClientWS.readyState;
                if (readyState === 1) {
                    clearInterval(ncWSConnectedWait);
                    connected = true;
                } else if (readyState !== 0) {
                    clearInterval(ncWSConnectedWait);
                    logger.error("NativeClient: Connection to js-bridge refused");
                }
            }.bind(this), 40);

            this.nativeClientWS.onmessage = function(evt) {
                var msg = evt.data;
                var jsonMessage = JSON.parse(evt.data);
                logger.trace("Received NativeClient Message " + jsonMessage.type + ", \"" + msg + "\"");
                switch (jsonMessage.type) {
                    case XRM_MSGTYPE_NATIVE_CLIENT_FEATURES:
                        var ncSuppEncList = jsonMessage[XRM_VCONF_LIST];
                        if (jsonMessage.hasOwnProperty(XRM_NATIVE_CLIENT_FEATURES)) {
                            var ncFeatures = jsonMessage[XRM_NATIVE_CLIENT_FEATURES];
                            ncLosslessEnabled = ncFeatures & XRM_SVR_PARAM_FEATURE_LOSSLESS;
                            ncVirtualHereSupport = ncFeatures & XRM_SVR_PARAM_FEATURE_VIRTUALHERE;
                        } else {
                            for (var i in ncSuppEncList) {
                                if (ncSuppEncList[i] === "9,125,8" || ncSuppEncList[i] === "6,6,8") {
                                    ncLosslessEnabled = true;
                                    break;
                                }
                            }
                        }
                        gPreferredVideoEncoding = jsonMessage[XRM_VCONF];
                        synchronized = true;
                        break;
                    case XRM_MSGTYPE_VH_DEVICE_LIST:
                        if (lastVHDeviceListMsg === msg)
                            break;
                        xrm_client.handleVHDeviceList(jsonMessage);
                        lastVHDeviceListMsg = msg;
                        break;
                    default:
                        logger.warn("Unrecognized NativeClient Message: " + jsonMessage.type);
                        break;
                }
            }.bind(this);
            return true;
        }
        return false;
    }.bind(this);

    this.send = function(data) {
        if (enabled === true && connected === true) {
            return this.nativeClientWS.send(data);
        }
        return false;
    }

    this.isEnabled = function() {
        return enabled;
    }

    this.isInitialized = function() {
        return initialized;
    }

    this.isConnected = function() {
        return connected;
    }

    this.isSynchronized = function() {
        return synchronized;
    }

    this.supportsLossless = function() {
        return ncLosslessEnabled;
    }

    this.supportsVirtualHere = function() {
        return ncVirtualHereSupport;
    }

    this.version = function() {
        return nativeClientVersion;
    }

    this.versionMajor = function() {
        return parseInt(nativeClientVersion);
    }

    ///// Do not delete
    this.constructor();
}

NativeClient.isNativeDecoderEnabled = function() {
    if (gNativeClientEnabled === true)
        return true;
    if (navigator.userAgent.search("Scyld Cloud Workstation Client".replaceAll(" ", "")) !== -1) {
        if (navigator.userAgent.search("decoder:WebGL") === -1) {
            gNativeClientEnabled = true;
            return true;
        }
    }
    return false;
}
