//
// Basic performance monitor on-screen display.
// Intended to only have 1 per browser window right now.
//
function PerformanceMonitor($parent, $toggle, id) {
    if (typeof PerformanceMonitor.count == 'undefined') {
        PerformanceMonitor.count = 0;
    }

    this.__$parent = $parent;
    this.__$toggle = $toggle;
    this.__id = id;

    if (this.__id === undefined || this.__id === null) {
        PerformanceMonitor.count += 1;
        this.__id = "performance-monitor-" + PerformanceMonitor.count;
    }

    this.__$parent.addClass("noselect");
    this.__$parent.addClass("text-left");
    this.__$parent.addClass("performance-monitor-container");

    this.__template =
        '  <a id="' + this.__id + '-container-close" type="button" class="close" aria-label="Close">&times;</a>' +
        '  <div id="' + this.__id + '-fps"><span class="performance-monitor-entry-lbl">fps</span><span id="' + this.__id + '-fps-value" class="performance-monitor-entry-value">...</span></div>' +
        '  <div id="' + this.__id + '-ping"><span class="performance-monitor-entry-lbl">ping</span><span id="' + this.__id + '-ping-value" class="performance-monitor-entry-value">...</span></div>' +
        '  <div id="' + this.__id + '-bandwidth"><span class="performance-monitor-entry-lbl">video bw</span><span id="' + this.__id + '-bandwidth-value" class="performance-monitor-entry-value">...</span></div>';

    //
    // Create widget and append to parent
    //
    this.__$parent.append(this.__template);

    this.__$fpsValue = $("#" + this.__id + "-fps-value");
    this.__$pingValue = $("#" + this.__id + "-ping-value");
    this.__$bandwidthValue = $("#" + this.__id + "-bandwidth-value");
    this.__active = this.__$parent.is(":visible");

    // Parse incoming stat message from server.
    // serverFrameCtr = myVideoServer.stats.frameCtr
    this.parseStatMessage = function(jsonMessage, currentFrameNumber) {
        if (this.__active === false && DEBUG_LEVEL < 1) {
            // No point in doing any of these calculations.
            return;
        }

        var serverClientFR = jsonMessage[XRM_SVR_STAT_REQUEST_FPI];
        var serverClientPing = jsonMessage[XRM_SVR_STAT_REQUEST_PING];
        var serverIntervalBR = jsonMessage[XRM_SVR_STAT_REQUEST_BPI];

        var bandwidth;
        if (serverIntervalBR > 131072) {
            bandwidth = (serverIntervalBR / 131072.0).toFixed(1) + " Mbps";
        } else {
            bandwidth = (serverIntervalBR / 128.0).toFixed(1) + " Kbps";
        }

        // Update performance values
        this.update(serverClientFR, serverClientPing + " ms", bandwidth);

        if (DEBUG_LEVEL >= 1) {
            var serverFrameNumber = jsonMessage[XRM_SVR_STAT_REQUEST_SVR_FRNO];
            var display = jsonMessage[XRM_MSG_SCREEN_ID];
            var serverClientCount = jsonMessage[XRM_SVR_STAT_REQUEST_CLIENT_COUNT];
            var serverIntervalSuppFR = jsonMessage[XRM_SVR_STAT_REQUEST_SUPP_FPI];
            var serverIntervalCurrFR = jsonMessage[XRM_SVR_STAT_REQUEST_CURR_FPI];

            // Update debug-specific client-stats
            var statusText = "Users: " + serverClientCount +
                ", FPS: " + serverClientFR +
                ", CliMaxFPS: " + serverIntervalCurrFR +
                ", SvrMaxFPS: " + serverIntervalSuppFR +
                ", VideoBW = " + bandwidth +
                ", Ping = " + serverClientPing + " ms";

            logger.trace("Stat Request: Frame: " + currentFrameNumber + " of " + serverFrameNumber + ", " + statusText);
        }

    }.bind(this);

    this.update = function(fps, ping, bandwidth) {
        this.__$fpsValue.text(fps);
        this.__$pingValue.text(ping);
        this.__$bandwidthValue.text(bandwidth);
    }.bind(this);

    // Close button
    $("#" + this.__id + "-container-close").on("click", function(e) {
        this.__$parent.hide();
        this.__active = false;
        this.__$toggle.find("span.far").removeClass("fa-toggle-on");
        this.__$toggle.find("span.far").addClass("fa-toggle-off");
    }.bind(this));

    // Menu-based toggle
    $toggle.on("click", function(e) {
        this.__$parent.toggle();
        if (this.__$parent.is(":visible")) {
            this.__active = true;
            this.update("...", "...", "...");
            this.__$toggle.find("span.far").removeClass("fa-toggle-off");
            this.__$toggle.find("span.far").addClass("fa-toggle-on");
        } else {
            this.__active = false;
            this.__$toggle.find("span.far").removeClass("fa-toggle-on");
            this.__$toggle.find("span.far").addClass("fa-toggle-off");
        }
    }.bind(this));


    return this;
}
