// General statistic handling

// Get more practical macro definitions
//

var StatsAvgWindow = 6;
var StatsStdWindow = 12;
var StatsAggrWindow = 48;

var LogConsole = true;

var LogInterval = 104;

function tStamps() {
    this.Entry = 0;
    this.Exit = 0;

    //copies members to from this to ts
    this.copy = function(ts) {
        ts.Entry = this.Entry;
        ts.Exit = this.Exit;
    };

    //prints out tstamp info related to ts (i.e. this - ts)
    this.print = function() {
        if (DEBUG_FRAMETIME) {
            logger.debug("Entry = ", this.Entry, "Exit = ", this.Exit);
        }
    };
}

function BaseStats(avg_win, std_win) {

    this.StatsAvgCnt = 0; // count of egated values
    this.StatsStdCnt = 0; // count of values used to calculate std dev

    this.StatsAvgWin = avg_win; // window of avg calculation
    this.StatsStdWin = std_win; // window of values used to calculate std dev

    this.StatsAvg = 0.0;  // average decode times
    this.StatsStd = 0.0;  // std var of decode times

    this.StatsDecodeStdMissedCnt = 0;  // count of over std values

    this.reinit = function(avg_win, std_win) {
        this.StatsAvgCnt = 0;
        this.StatsStdCnt = 0;

        this.StatsAvgWin = avg_win;
        this.StatsStdWin = std_win;

        this.StatsAvg = 0.0;
        this.StatsStd = 0.0;
    };

    this.calc = function(value, calc_std) {
        if (typeof(calc_std) === "undefined") {
            calc_std = true; // by default calculate std/var as well
        }

        if (this.StatsAvgWin == this.StatsAvgCnt) { // Sliding window over StatsAvgCnt measurements
            this.StatsAvgCnt = this.StatsAvgWin;
        } else {
            this.StatsAvgCnt = this.StatsAvgCnt + 1;
        }

        this.StatsAvg = (this.StatsAvg * (this.StatsAvgCnt - 1) + value) / (this.StatsAvgCnt);

        if (!calc_std) {
            return;
        }

        // Calculate var:

        if (this.StatsStdCnt > this.StatsAvgWin) { // We have a proper average

            if (this.StatsStdCnt >= this.StatsStdWin) { // ... and are sliding over var
                this.StatsStdCnt = this.StatsStdCnt - 1;
            }

            var act_var_val = (this.StatsAvg - value) * (this.StatsAvg - value);
            var act_var_count = this.StatsStdCnt - this.StatsAvgWin;

            var act_var_delta = act_var_count / (act_var_count + 1);

            this.StatsStd = (this.StatsStd * act_var_delta + act_var_val) / (act_var_count + 1);

        }

        this.StatsStdCnt = this.StatsStdCnt + 1;

    };

    this.avgResultValid = function() {
        return this.StatsAvgCnt == this.StatsAvgWin;
    };

    this.stdResultValid = function() {
        return this.StatsStdCnt > this.StatsAvgWin;
    };

    this.getAvgValue = function(log_precision) {
        if (typeof(log_precision) === "undefined") {
            log_precision = 4; // by default calculate std/var as well
        }

        if (this.avgResultValid()) {
            if (log_precision >= 0) {
                return this.StatsAvg.toFixed(log_precision);
            } else {
                return this.StatsAvg;
            }
        } else {
            return -1;
        }
    };

    this.getAvgValueForced = function(log_precision) {
        if (typeof(log_precision) === "undefined") {
            log_precision = 4; // by default calculate std/var as well
        }

        if (this.StatsAvgCnt > 0) {
            if (log_precision >= 0) {
                return this.StatsAvg.toFixed(log_precision);
            } else {
                return this.StatsAvg;
            }
        } else {
            return -1;
        }
    };

    this.getStdValue = function(log_precision) {
        if (typeof(log_precision) === "undefined") {
            log_precision = 4; // by default calculate std/var as well
        }

        if (this.stdResultValid()) {
            if (log_precision >= 0) {
                return Math.sqrt(this.StatsStd).toFixed(log_precision);
            } else {
                return Math.sqrt(this.StatsStd);
            }
        } else {
            return -1;
        }
    };
}

function Stats() {

    this.targetFrameTime = 1.0;
    this.configNo = 0;
    this.frameCtr = 0;
    this.frameCtrIntStart = Date.now();
    this.frameCtrCurrInt = 0;
    this.frameCtrPrevInt = 0;
    this.decodeTimeStats = new BaseStats(StatsAvgWindow, StatsStdWindow);
    this.StatsReportConsoleWin = LogInterval + 1; // report every x frame
    this.StatsReportConsoleCnt = 0; // report frame counter
    this.StatsPrevFrame = Date.now(); // ts of previous frame received

    this.m_frameTimems = 0;
    this.m_decodeTimems = 0;

    if (DEBUG_LEVEL < 8) {
        this.log = function(){};
    } else {
        this.log = function() {
            var avg_value = this.decodeTimeStats.getAvgValue(4);
            var std_value = this.decodeTimeStats.getStdValue(4);
            logger.debug("Decode avg:" + avg_value + " / std:" + std_value);
        };
    }

    this.reinit = function() {

        LogInterval = LogIntervalSec * 1000 / this.targetFrameTime;

        this.decodeTimeStats.reinit(StatsAvgWindow, StatsStdWindow);
        this.StatsReportConsoleWin = LogInterval + 1;
        this.StatsReportConsoleCnt = 0;
        this.StatsPrevFrame = 0;
        this.m_frameTimems = 0;
        this.m_decodeTimems = 0;

        logger.debug("Target time per frame = " + this.targetFrameTime + ", LogInterval = " + LogInterval);
    };

    this.calc = function(ts_frame, size) {
        this.m_frameTimems = ts_frame.Entry - this.StatsPrevFrame;
        this.m_decodeTimems = ts_frame.Exit - ts_frame.Entry;

        this.decodeTimeStats.calc(this.m_decodeTimems);

        this.StatsPrevFrame = ts_frame.Entry;

        this.StatsReportConsoleCnt = this.StatsReportConsoleCnt + 1;

        if (LogConsole && (this.StatsReportConsoleCnt >= this.StatsReportConsoleWin)) {
            this.log();
            this.StatsReportConsoleCnt = 1;
        }
        return false;
    };

    this.sendStats = function(p_sendws, serverFrameNumber, screenID) {

        var backlog = serverFrameNumber - this.frameCtr;

        if (backlog < 0) {
            backlog = 0;
        }

        var obj = {};
        obj[XRM_MSG_TYPE] = XRM_MSGTYPE_CLNT_STATS;
        obj[XRM_MSG_SCREEN_ID] = screenID;
        obj[XRM_SVR_VS_PARAM_CONFIG_NO] = this.configNo;
        obj[XRM_CLNT_STATS_CLNT_FRAMENO] = this.frameCtr;
        obj[XRM_CLNT_STATS_SVR_FRAMENO] = serverFrameNumber;
        obj[XRM_CLNT_STATS_CLNT_FRPINT] = this.frameCtrPrevInt;
        obj[XRM_CLNT_STATS_BACKLOG] = backlog;
        obj[XRM_CLNT_STATS_DECODE_AVG] = this.decodeTimeStats.getAvgValue(4);
        obj[XRM_CLNT_STATS_DECODE_STD] = this.decodeTimeStats.getStdValue(4);

        var msg = JSON.stringify(obj);

        logger.debug("Stats send: " + msg);

        p_sendws.send(msg);
    };
}
