function UsbDevice(parent, address, path, name, productId, vendorId, state, websocket) {
    this.parent = parent;
    this.address = address;
    this.elementId = "usbdev-" + address;

    this.path = path;
    this.name = name;
    this.productId = productId;
    this.vendorId = vendorId;
    this.state = state;
    this.__websocket = websocket;

    this.setPath = function(path) {
        this.path = path;
    }

    this.setName = function(name) {
        this.name = name;
    }

    this.setProductId = function(productId) {
        this.productId = productId;
    }

    this.setVendorId = function(vendorId) {
        this.vendorId = vendorId;
    }

    this.setState = function(state) {
        this.state = state;
    }

    this.draw = function() {
        var statusIcon = "far fa-toggle-off";
        var disabled = false;
        var title = "";

        switch(this.state) {
            case VH_DEVICE_DISCOVERED:
                // device found, not ready for request
                disabled = true;
                statusIcon = "far fa-hourglass-half";
                title = "Searching..."
                break;
            case VH_DEVICE_EXPORTED:
                // device found, ready for requested
                disabled = false;
                statusIcon = "far fa-toggle-off";
                break;
            case VH_DEVICE_BIND_REQUESTED:
                // device requested
                disabled = true;
                statusIcon = "far fa-hourglass-half";
                title = "Connecting..."
                break;
            case VH_DEVICE_BOUND:
                // device bound
                disabled = false;
                statusIcon = "far fa-toggle-on";
                break;
            case VH_DEVICE_RELEASE_REQUESTED:
                // device released
                disabled = true;
                statusIcon = "far fa-hourglass-half";
                title = "Disconnecting..."
                break;
            case VH_DEVICE_IGNORED:
                // device is ignored (ex: hubs)
                disabled = true;
                return null;
            case VH_DEVICE_ERROR:
                // error state
                disabled = true;
                statusIcon = "far fa-exclamation-triangle";
                title = "Unexpected device error."
                break;
            default:
                break;
        }

        var label = this.name;
        var html = "<li id='" + this.elementId + "' title='" + title + "'><a class='usbdev dropdown-item' href='#'><span class='" + statusIcon + " fa-fw fa-lg'></span> <span id='label-" + this.elementId + "'>" + label + "</span></a></li>";

        if ($("#" + this.elementId).length) {
            $("#" + this.elementId).replaceWith(html);
        } else {
            $(parent).append(html);
        }

        var element = $("#" + this.elementId);
        if (disabled) {
            element.addClass("disabled");
        } else {
            element.removeClass("disabled");
        }

        element.on("click", function(e) {
            var obj = {}
            switch(this.state) {
                case VH_DEVICE_EXPORTED:
                    // device found, ready for requested
                    obj[XRM_MSG_TYPE] = XRM_MSGTYPE_VH_CONNECT_DEVICE;
                    obj[XRM_MSG_VH_DEV_ADDRESS] = address;
                    this.__websocket.send(JSON.stringify(obj));

                    // show user that we requested a connect
                    this.setState(VH_DEVICE_BIND_REQUESTED);
                    this.draw();
                    break;
                case VH_DEVICE_BIND_REQUESTED:
                    // device requested
                    break;
                case VH_DEVICE_BOUND:
                    // device bound
                    obj[XRM_MSG_TYPE] = XRM_MSGTYPE_VH_DISCONNECT_DEVICE;
                    obj[XRM_MSG_VH_DEV_ADDRESS] = address;
                    this.__websocket.send(JSON.stringify(obj));

                    // show user that we requested a disconnect
                    this.setState(VH_DEVICE_RELEASE_REQUESTED);
                    this.draw();
                    break;
                case VH_DEVICE_RELEASE_REQUESTED:
                    // device released
                    break;
                case VH_DEVICE_IGNORED:
                    // device is ignored (ex: hubs)
                    break;
                case VH_DEVICE_ERROR:
                    // error state
                    break;
                default:
                    break;
            }
            e.stopPropagation(e);
        }.bind(this));
    }

}
