// Example:
// new UserButton(document.getElementById("user-bar"), clientID, clientName, isHost, hasVideo, hasControl, ip, port, isMe, myIsHost, dataWS);
function UserButton(parent, clientID, name, isHost, hasVideo, hasControl, ip, port, isMe, myIsHost, websocket) {
    var SLOW_FRAMERATE_FACTOR = 2;
    var STATUS_CLASSES = {
        "host": "user-status-host",
        "online": "fas fa-circle user-status-online",
        "paused": "fas fa-pause user-status-paused",
        "slow": "fas fa-tachometer-alt-slow user-status-slow",
        "controller": "fas fa-keyboard fa-lg user-status-controller",
        "controller-paused": "fas fa-keyboard fa-lg user-status-paused",
    };

    this.__button = null;
    this.__parent = parent;
    this.clientID = clientID;
    this.__name = name;
    this.__isHost = isHost;
    this.__hasVideo = hasVideo;
    this.__hasControl = hasControl;
    this.__ip = ip;
    this.__port = port;
    this.__isMe = isMe;
    this.__frameRate = "...";
    // supportedFrameRate is the max FPS this client can support right now under ideal network conditions
    this.__supportedFrameRate = "...";
    this.__svrSupportedFrameRate = "...";
    this.__websocket = websocket;
    this.__consecutiveSlowUpdates = 0;

    this.remove = function() {
        if ($("#user-" + this.clientID).length) {
            $("#user-" + this.clientID).remove();
            return true;
        } else {
            return false;
        }
    }

    this.draw = function() {
        var actions = "";

        if (this.__isMe) {
            // Actions a host can perform on him/herself
            if (myIsHost) {
                actions += '<li><a class="user-action take-control" href="#">Take keyboard and mouse</a></li>';
            }
        } else {
            if (myIsHost) {
                // Actions a host can perform on a guest
                if (this.__isHost === false) {
                    actions += '<li><a class="user-action kick-guest" href="#">Kick</a></li>';
                }
                // Actions a host can perform on anyone
                actions += '<li><a class="user-action give-control" href="#">Give keyboard and mouse</a></li>';
            }
        }

        var template =
            '<div id="user-' + this.clientID + '" class="btn-group user-button-group">' +
            '  <div class="btn-group dropup">' +
            '    <a type="button" class="btn btn-md btn-default dropdown-toggle user-button" data-toggle="dropdown">' +
            '      <span class="user-icon"></span>' +
            '      <span class="user-name"></span>' +
            '      <span class="user-icon2"></span>' +
            '    </a>' +
            '    <ul class="dropdown-menu user-info-container" role="menu">' +
            '      <li class="disabled"><a class="user-info" href="#">Status<span class="pull-right user-status"></span></a></li>' +
            '      <li class="disabled"><a class="user-info" href="#">Address<span class="pull-right user-address"></span></a></li>' +
            '      <li class="disabled"><a class="user-info" href="#">Frame Rate(s)<span class="pull-right user-frameRate"></span> <span class="user-icon2"></span></a></li>' +
            '      <li role="separator" class="divider"></li>' +
            actions +
            '    </ul>' +
            '  </div>' +
            '</div>';

        if ($("#user-" + this.clientID).length) {
            $("#user-" + this.clientID).replaceWith(template);
        } else {
            $(parent).append(template);
        }

        var $btn = $("#user-" + this.clientID);
        if (actions === "") {
            $btn.find(".divider:first").hide();
        }

        $btn.find(".user-info").on("click", function() {
            return false;
        }.bind(this));

        $btn.find(".kick-guest").on("click", function() {
            var cid = this.clientID;
            var obj = {};
            obj[XRM_MSG_TYPE] = XRM_MSGTYPE_MU_CLNT_DISCONNECT_CLIENT;
            obj[XRM_MU_CLNT_DISCONNECT_CLIENT_CID] = cid;
            this.__websocket.send(JSON.stringify(obj));
        }.bind(this));

        $btn.find(".take-control").on("click", function() {
            var cid = this.clientID;
            var obj = {};
            obj[XRM_MSG_TYPE] = XRM_MSGTYPE_MU_CLNT_KBM_CTRL;
            obj[XRM_MU_CLNT_KBM_CTRL_CID] = cid;
            obj[XRM_MU_CLNT_KBM_CTRL_VAL] = 1;
            this.__websocket.send(JSON.stringify(obj));
        }.bind(this));

        $btn.find(".give-control").on("click", function() {
            var cid = this.clientID;
            var obj = {};
            obj[XRM_MSG_TYPE] = XRM_MSGTYPE_MU_CLNT_KBM_CTRL;
            obj[XRM_MU_CLNT_KBM_CTRL_CID] = cid;
            obj[XRM_MU_CLNT_KBM_CTRL_VAL] = 1;
            this.__websocket.send(JSON.stringify(obj));
        }.bind(this));

        this.__button = $("#user-" + this.clientID);
        this.updateStatus();
    }.bind(this);

    this.setID = function(id) {
        this.clientID = id;
    }.bind(this);

    this.setName = function(name) {
        this.__name = name;
        var SHORT_NAME_LEN = 12;

        this.__shortName = this.__name;
        if (this.__isMe) {
            this.__shortName = "You";
        } else if (this.__name.length > SHORT_NAME_LEN) {
            this.__shortName = htmlEncode(this.__name.substring(0, SHORT_NAME_LEN) + '...');
        }
    }.bind(this);

    this.setIP = function(ip) {
        this.__ip = ip;
    }.bind(this);

    this.setPort = function(port) {
        this.__port = port;
    }.bind(this);

    this.setIsHost = function(isHost) {
        this.__isHost = isHost;
    }.bind(this);

    this.setHasVideo = function(hasVideo) {
        this.__hasVideo = hasVideo;
    }.bind(this);

    this.setHasControl = function(hasControl) {
        this.__hasControl = hasControl;
    }.bind(this);

    this.setFrameRate = function(fr) {
        if (this.__frameRate === "..." && (fr === "0" || fr === "")) {
            return; // don't update to a bogus frame rate
        }
        this.__frameRate = fr;
    }.bind(this);

    this.setSupportedFrameRate = function(fr) {
        if (this.__supportedFrameRate === "..." && (fr === "0" || fr === "")) {
            return; // don't update to a bogus frame rate
        }
        this.__supportedFrameRate = fr;
    }.bind(this);

    this.setSvrSupportedFrameRate = function(fr) {
        this.__svrSupportedFrameRate = fr;
    }.bind(this);

    this.clientPerformanceOk = function() {
        var supportedFrameRates = this.__supportedFrameRate.split(",");
        var svrSupportedFrameRates = this.__svrSupportedFrameRate.split(",");
        var slowRateFound = false;

        if (this.__supportedFrameRate === "..." || this.__svrSupportedFrameRate == "...")
            return true;

        for (var i = 0; i < supportedFrameRates.length && i < svrSupportedFrameRates.length; i++) {
            if (!isNaN(parseInt(supportedFrameRates[i])) && !isNaN(parseInt(svrSupportedFrameRates[i]))) {
                if (parseInt(supportedFrameRates[i]) * SLOW_FRAMERATE_FACTOR < parseInt(svrSupportedFrameRates[i])) {
                    slowRateFound = true;
                }
            }
        }
        return !slowRateFound;
    }.bind(this);

    /**
     * Online = icon -> green dot
     * Controller = icon -> green keyboard
     * Paused = icon -> gray pause
     * Paused-Controller = icon -> gray keyboard
     * Slow = icon2 -> orange hourglass
     * Host = name -> bold
     */
    this.updateStatus = function() {
        var $btn = this.__button;

        // set ID
        $btn.find(".user-id:first").text(this.clientID);

        // set Name
        $btn.find(".user-name:first").text(htmlEncode(this.__shortName));
        $btn.find(".btn:first").attr("title", htmlEncode(this.__name));

        // set Address
        $btn.find(".user-address:first").text(this.__ip + ':' + this.__port);

        // set frame rate
        $btn.find(".user-frameRate:first").text(this.__frameRate);

        var user_icon = $btn.find(".user-icon:first");
        var user_icon2 = $btn.find(".user-icon2");
        var user_name = $btn.find(".user-name:first");

        $btn.find(".btn:first").addClass("btn-default");
        $btn.find(".btn:first").removeClass("btn-faded");
        user_icon.attr("class", "user-icon");
        user_name.attr("class", "user-name");
        user_icon2.attr("class", "user-icon2");

        if (this.__isHost === true) {
            this.__status = "Host";
            user_name.addClass(STATUS_CLASSES["host"]);
            user_name.text(this.__shortName + "*");
            user_icon.addClass(STATUS_CLASSES["online"]);
        } else {
            this.__status = "Guest";
            if (this.__hasVideo === false) {
                this.__status += ", Paused";
                user_icon.addClass(STATUS_CLASSES["paused"]);
                $btn.find(".btn:first").removeClass("btn-default");
                $btn.find(".btn:first").addClass("btn-faded");
                user_name.addClass("text-muted");
            } else {
                user_icon.addClass(STATUS_CLASSES["online"]);
            }
        }

        if (this.__hasControl === true) {
            this.__status += ", Controller";
            user_icon.attr("class", "user-icon");
            if (this.__hasVideo)
                user_icon.addClass(STATUS_CLASSES["controller"]);
            else
                user_icon.addClass(STATUS_CLASSES["controller-paused"]);
        }

        if (this.__hasVideo && !this.clientPerformanceOk()) {
            if (this.__consecutiveSlowUpdates > 1) {
                this.__status += ", Slow";
                user_icon2.addClass(STATUS_CLASSES["slow"]);
            }
            this.__consecutiveSlowUpdates += 1;
        } else {
            user_icon2.removeClass(STATUS_CLASSES["slow"]);
            this.__consecutiveSlowUpdates = 0;
        }

        $btn.find(".user-status:first").text(this.__status);

        if (myIsHost) {
            if (this.__isMe) {
                if (this.__hasControl) {
                    $btn.find(".divider").hide();
                    $btn.find(".take-control").hide();
                } else {
                    $btn.find(".divider").show();
                    $btn.find(".take-control").show();
                }
            } else {
                if (this.__hasControl) {
                    if (this.__isHost) {
                        $btn.find(".divider").hide();
                    }
                    $btn.find(".give-control").hide();
                } else {
                    $btn.find(".divider").show();
                    $btn.find(".give-control").show();
                }

                /**
                 * Hosts can't give Input Control to users who are
                 * paused so don't display request button.
                 */
                if (this.__hasVideo == false) {
                    $btn.find(".give-control").hide();
                }
            }
        }
    }.bind(this);

    this.setName(name);
    this.setIsHost(isHost);
    this.setHasVideo(hasVideo);
    this.setHasControl(hasControl);
    this.setIP(ip);
    this.setFrameRate("...");
    this.setSupportedFrameRate("...");

    return this;
}
