//
// Defines important information that describes the state of the client
//
// DEPENDS ON:
//  * bowser.js
//  * xrm-utils.js
//  * native-client.js
//

if (typeof bowser === 'undefined') {
    alert("bowser was not defined before xrm-globals.js attempted to run");
    console.log("bowser was not defined before xrm-globals.js attempted to run");
}

assertRequirements();
updateClientPlatformInfo();

//
// Detect if 'passive' events are supported by this browser
//
var passiveSupported = false;
try {
    var passiveCheckOptions = {};

    // This function will be called when the browser
    // attempts to access the passive property.
    Object.defineProperty(passiveCheckOptions, "passive", {
        get: function() { passiveSupported = true; }
    });

    window.addEventListener("test", passiveCheckOptions, passiveCheckOptions);
    window.removeEventListener("test", passiveCheckOptions, passiveCheckOptions);
} catch(err) {
    passiveSupported = false;
}
delete passiveCheckOptions;

//
// Test for required things: Websocket, WebGL, etc.  Assumption is
// that if something is missing, we do not continue - we redirect to
// an error handling page.
//
// Sets two globals - rendermode and webGLSupported
//
var rendermode;
var webGLSupported;
function assertRequirements() {
    // test websocket support
    if (!("WebSocket" in window)) {
        document.location.href = "/no-websocket.html";
    }

    var testWebGLSupport = function() {
        var canvasgltest = document.createElement("canvas");
        var gltest = null;
        try {
            gltest = canvasgltest.getContext("webgl");
        } catch (x) {
            gltest = null;
        } // nope, no webgl

        // experimental webgl?
        if (gltest == null) {
            try {
                gltest = canvasgltest.getContext("experimental-webgl");
            } catch (x) {
                gltest = null;
            } // nope, not even experimental-gl
        }
        return gltest !== null;
    }
    var webgl = testWebGLSupport();

    // get URL parameter for webgl
    var render = getURLParameter("rendermode");
    if (render != "nowebgl") {
        render = "webgl";
    }

    // if rendermode is webgl, check webgl support otherwise just go ahead and start
    if (NativeClient.isNativeDecoderEnabled() === false && render == "webgl") { // Native client change
        if (webgl === false) {
            document.location.href = "/no-webgl.html";
        }
    }

    rendermode = render;
    webGLSupported = webgl;
}

//
// Set global values clientOS, clientOSversion, and update global bowser
//
var clientOS;
var clientOSversion;
function updateClientPlatformInfo() {
    clientOS = bowser.osversion;
    clientOSversion = bowser.osversion;

    if (navigator.userAgent.search("Scyld Cloud Workstation Client".replaceAll(" ", "")) !== -1) {
        bowser.name = "Scyld Cloud Workstation Client";
        var clVer = navigator.userAgent.substring(0, navigator.userAgent.indexOf(" "));
        bowser.version = clVer.substring(clVer.indexOf("/") + 1);
    }

    if (bowser.ios) {
        clientOS = "IOS";
    } else if (bowser.android) {
        clientOS = "Android";
    } else if (bowser.windowsphone) {
        clientOS = "Windows Phone";
    } else if (bowser.blackberry) {
        clientOS = "BlackBerry";
    } else if (navigator.userAgent.search(/Windows NT/i) !== -1) {
        clientOS = "Windows";
        clientOSversion = navigator.userAgent; // "Pre-XP"
        var WindowsVersions = [
            {wv:"XP", r:/(Windows NT 5.1|Windows XP)/},
            {wv:"2003", r:/Windows NT 5.2/},
            {wv:"Vista", r:/Windows NT 6.0/},
            {wv:"7", r:/(Windows 7|Windows NT 6.1)/},
            {wv:"8", r:/(Windows 8|Windows NT 6.2)/},
            {wv:"8.1", r:/(Windows 8.1|Windows NT 6.3)/},
            {wv:"10", r:/(Windows NT 10|Windows NT 6.4)/}
        ];
        for (var id in WindowsVersions) {
            var cwv = WindowsVersions[id];
            if (cwv.r.test(navigator.userAgent)) {
                clientOSversion = cwv.wv;
                break;
            }
        }
    } else if (navigator.userAgent.search(/Linux/i) !== -1) {
        clientOS = "Linux";
    } else if (navigator.userAgent.search(/Mac OS/i) !== -1 || navigator.userAgent.search(/Macintosh/i) !== -1) {
        clientOS = "MacOS";
    } else if (navigator.userAgent.search(/SunOS/i) !== -1) {
        clientOS = "SunOS";
    } else {
        clientOS = navigator.userAgent;
    }
    return { 'os': clientOS, 'osVersion': clientOSversion, 'bowser': bowser };
}

//
// Screen ID
//
var myScreenID = parseInt(getURLParameter("screen"));
if (isNaN(myScreenID)) {
    myScreenID = 0;
}

/* Custom scrollbar */
var myScrollbars = OverlayScrollbars(document.body,
                                     {
                                         className : "os-theme-thick-dark",
                                         scrollbars: { autoHide : 'move' },
                                     });

// Changing of the following two variables (by the native client) only has affect 
// with single window web clients as screen-window has an independant copy of the
// variables
var gPreferredVideoEncoding = "0,0,8"; // lossless = "9,125,8";

var x11ServerFeatures = 0; // = XRM_SVR_PARAM_FEATURE_NONE;
