//
// Wrapper function for WebSocket.  Allows us to pass it around by reference before establishing the connection.
//
// JS DEPENDENCIES: None.
//
function XrmWebSocket() {
    var webSocket = null;
    var errorFlag = false;

    this.isInit = function() {
        return webSocket === null;
    }

    this.open = function(url) {
        logger.debug("websocket url = " + url);
        webSocket = new WebSocket(url);

        webSocket.onopen = function(e) {
            this.onopen(e);
            $.ajax({url: "resource-ticket",
                    method: "POST",
                    cache: false,
                    data: {"uri": url, "CSRFToken": $("#CSRFToken").val()},
                    dataType: "json",
                    context: this,
                    success: function(data) {
                        if (data.success === false) {
                            logger.error("Failed to get resource-ticket for WebSocket.");
                            return false;
                        }
                        var msg = JSON.stringify(data);
                        this.send(msg);
                        this.onauth(e);
                    },
                    error: function(data) {
                        if (data.success === false) {
                            logger.error("Failed to get resource-ticket for WebSocket.");
                            return false;
                        }
                    }});
        }.bind(this);

        webSocket.onerror = function(e) {
            errorFlag = true;
            this.onerror(e);
        }.bind(this);

        webSocket.onmessage = function(e) {
            this.onmessage(e);
        }.bind(this);

        webSocket.onclose = function(e) {
            if (e.code === 1006 && errorFlag) {
                logger.error("Error: Websocket closed unexpectedly.");
            }
            this.onclose(e);
        }.bind(this);

        // dummy placeholders to prevent 'undefined' errors -- leave empty
        this.onopen = function() {};
        this.onauth = function() {};
        this.onerror = function() {};
        this.onmessage = function() {};
        this.onclose = function() {};

        return this;
    }

    this.close = function(code, reason) {
        return webSocket.close(code, reason);
    }

    this.send = function(data) {
        return webSocket.send(data);
    }

    this.setBinaryType = function(v) {
        webSocket.binaryType = v;
    }

    this.getBinaryType = function() {
        return webSocket.binaryType;
    }

    this.getBufferedAmount = function() {
        return webSocket.bufferedAmount;
    }

    this.getExtensions = function() {
        return webSocket.extensions;
    }

    this.getProtocol = function() {
        return webSocket.protocol;
    }

    this.getReadyState = function() {
        return webSocket.readyState;
    }

    this.getURL = function() {
        return webSocket.url;
    }
}

//
// The default protocol depends on the location.protocol.
// If we're on HTTP, we use WS.
// If we're on HTTPS, we use WSS.
//
XrmWebSocket.defaultProtocol = function() {
    if (location.protocol == "http:") {
        return "ws://";
    } else {
        return "wss://";
    }
}
